-- Setting starting directory for filebrowser
System.currentDirectory("/")

-- Optimized generic print function for code which needs to be executed only one time
function OneshotPrint(my_func)
	my_func()
	Screen.flip()
	Screen.refresh()
	my_func()
end

-- CPU related settings
cpu_speed = 268

-- Functions used by filebrowser
function CropPrint(x, y, text, color, screen)
	if string.len(text) > 25 then
		Screen.debugPrint(x, y, string.sub(text,1,25) .. "...", color, screen)
	else
		Screen.debugPrint(x, y, text, color, screen)
	end
end
function TableConcat(t1,t2)
    for i=1,#t2 do
        t1[#t1+1] = t2[i]
    end
    return t1
end
function listDirectory(dir)
	dir = System.listDirectory(dir)
	folders_table = {}
	files_table = {}
	for i,file in pairs(dir) do
		if file.directory then
			table.insert(folders_table,file)
		elseif (string.lower(string.sub(file.name,-4)) == ".obj") then
			if string.sub(file.name,1,1) ~= "." then
				table.insert(files_table,file)
			end
		end
	end
	table.sort(files_table, function (a, b) return (a.name:lower() < b.name:lower() ) end)
	table.sort(folders_table, function (a, b) return (a.name:lower() < b.name:lower() ) end)
	return_table = TableConcat(folders_table,files_table)
	return return_table
end
function OpenDirectory(text)
	i=0
	if text == ".." then
		j=-2
		while string.sub(System.currentDirectory(),j,j) ~= "/" do
			j=j-1
		end
		System.currentDirectory(string.sub(System.currentDirectory(),1,j))
	else
		System.currentDirectory(System.currentDirectory()..text.."/")
	end
	files_table = listDirectory(System.currentDirectory())
	if System.currentDirectory() ~= "/" then
		local extra = {}
		extra.name = ".."
		extra.size = 0
		extra.directory = true
		table.insert(files_table,extra)
	end
end

-- Starting filebrowser
files_table = listDirectory(System.currentDirectory())
master_index = 0
p = 1
oldpad = 0

-- Creating a blank texture for non-textured models
tmp = Screen.createImage(64, 64, Color.new(195,195,195))
no_texture = Graphics.convertFrom(tmp)
Screen.freeImage(tmp)
has_texture = false

-- Setting default vertex shader and material attributes
light_color = Render.createColor(1.0, 1.0, 1.0, 1.0)
ambient = Render.createColor(0.2, 0.2, 0.2, 0.0)
diffuse = Render.createColor(0.4, 0.4, 0.4, 0.0)
specular = Render.createColor(0.8, 0.8, 0.8, 0.0)
local background_top = Color.new(115,115,115) -- original Blender background color
Render.init(400, 240, background_top)
Render.setLightColor(light_color)
Render.setLightSource(0.5, 0.5, -0.5)

-- Init a couple of colors
local black = Color.new(0,0,0)
local white = Color.new(255,255,255)
local red = Color.new(255, 0, 0)
local yellow = Color.new(255, 255, 0)

-- Loading alert function
function Alert()
	Screen.fillRect(0, 399, 0, 239, background_top, TOP_SCREEN)
	Screen.debugPrint(120, 110, "Loading model...", black, TOP_SCREEN)
end

-- Init default model positioning
z = -40.0
x = 0.0
y = 0.0
angleX = 0.0
angleY = 0.0

-- Init timer for fps calculation purposes
tmr = Timer.new()
fps = 0
fps_sec = 0

-- Enabling stereoscopic 3D
Screen.refresh()
Screen.enable3D()
if Screen.get3DLevel() > 0.0 then
	is3D = true
else
	is3D = false
end

-- Main loop
while true do
		
	-- Reading input
	pad = Controls.read()
	
	-- Updating is3D flag
	if Screen.get3DLevel() > 0.0 then
		is3D = true
	else
		is3D = false
	end
	
	-- Increasing FPS counter
	fps = fps + 1
	
	-- Drawing model
	if renderPhase then
		Render.initBlend(TOP_SCREEN, LEFT_EYE)
		Render.drawModel(mod1, x + Screen.get3DLevel(), y, z, angleX, angleY)
		Render.termBlend()
		if is3D then
			Render.initBlend(TOP_SCREEN, RIGHT_EYE)
			Render.drawModel(mod1, x - Screen.get3DLevel(), y, z, angleX, angleY)
			Render.termBlend()
		end
	else
		Screen.fillRect(0, 399, 0, 239, background_top, TOP_SCREEN)
	end
	
	-- Updating bottom screen
	Screen.refresh()
	Screen.waitVblankStart()	
	Screen.clear(BOTTOM_SCREEN)
	
	if renderPhase then
	
		-- Drawing model info and fps
		Screen.debugPrint(2, 2, "X: " .. x, white, BOTTOM_SCREEN)
		Screen.debugPrint(2, 17, "Y: " .. y, white, BOTTOM_SCREEN)
		Screen.debugPrint(2, 32, "Z: " .. z, white, BOTTOM_SCREEN)
		Screen.debugPrint(2, 47, "Angle X: " .. angleX, white, BOTTOM_SCREEN)
		Screen.debugPrint(2, 62, "Angle Y: " .. angleY, white, BOTTOM_SCREEN)
		Screen.debugPrint(2, 208, "CPU: " .. cpu_speed .. " Mhz", white, BOTTOM_SCREEN)
		Screen.debugPrint(2, 223, "FPS: " .. fps_sec, white, BOTTOM_SCREEN)
		
		-- Closing Model Viewer
		if Controls.check(pad, KEY_B) then
			renderPhase = false
			Render.unloadModel(mod1)
			if hasTexture then
				Graphics.freeImage(texture)
			end
		end
		
	else
	
		-- Drawing Filebrowser
		base_y = 0
		for l, file in pairs(files_table) do
			if (base_y > 226) then
				break
			end
			if (l >= master_index) then
				if (l==p) then
					base_y2 = base_y
					if (base_y) == 0 then
						base_y = 2
					end
					Screen.fillRect(0,319,base_y-2,base_y2+12,yellow,BOTTOM_SCREEN)
					color = red
					if (base_y) == 2 then
						base_y = 0
					end
				else
					color = white
				end
				CropPrint(0,base_y,file.name,color,BOTTOM_SCREEN)
				base_y = base_y + 15
			end
		end
		
		-- Filebrowser navigation
		if (Controls.check(pad,KEY_A) and not Controls.check(oldpad,KEY_A)) then
			if (files_table[p].directory) then
				OpenDirectory(files_table[p].name)
				p = 1
				master_index = 0
			else
				if renderPhase then
					Render.unloadModel(mod1)
					if hasTexture then
						Graphics.freeImage(texture)
					end
				end
				OneshotPrint(Alert)
				renderPhase = true
				tex_name = string.sub(files_table[p].name,1,string.len(files_table[p].name)-4) .. ".png"
				if System.doesFileExist(System.currentDirectory()..tex_name) then
					texture = Graphics.loadImage(System.currentDirectory()..tex_name)
					hasTexture = true
					mod1 = Render.loadObject(System.currentDirectory()..files_table[p].name, texture, ambient, diffuse, specular, 1.0)
				else
					hasTexture = false
					mod1 = Render.loadObject(System.currentDirectory()..files_table[p].name, no_texture, ambient, diffuse, specular, 1.0)
				end
			end
		elseif (Controls.check(pad,KEY_DUP)) and not (Controls.check(oldpad,KEY_DUP)) then
			p = p - 1
			if (p >= 16) then
				master_index = p - 15
			end
		elseif (Controls.check(pad,KEY_DLEFT)) and not (Controls.check(oldpad,KEY_DLEFT)) then
			p = p - 16
			if p < 1 then
				p = 1
			end
			if (p >= 16) then
				master_index = p - 15
			else
				master_index = 0
			end
		elseif (Controls.check(pad,KEY_DRIGHT)) and not (Controls.check(oldpad,KEY_DRIGHT)) then
			p = p + 16
			if p > #files_table then
				p = #files_table
			end
			if (p >= 17) then
				master_index = p - 15
			end
		elseif (Controls.check(pad,KEY_DDOWN)) and not (Controls.check(oldpad,KEY_DDOWN)) then
			p = p + 1
			if (p >= 17) then
				master_index = p - 15
			end
		end
		if p < 1 then
			p = #files_table
			if p >= 17 then
				master_index = p - 15
			end
		elseif p > #files_table then
			master_index = 0
			p = 1
		end
		
	end
	
	-- Exiting homebrew
    if Controls.check(pad, KEY_START) then
		if renderPhase then
			Render.unloadModel(mod1)
			if hasTexture then
				Graphics.freeImage(texture)
			end
		end
		Graphics.freeImage(no_texture)
		Render.term()
		Timer.destroy(tmr)
		Screen.disable3D()
        System.exit()
    end
	
	-- Model movement
	if renderPhase then
		-- Screenshot function
		if Controls.check(pad, KEY_SELECT) then
			h,m,s = System.getTime()
			day_value,day,month,year = System.getDate()
			System.takeScreenshot(System.currentDirectory().."/screenshot_"..string.format("%02d", day)..string.format("%02d", month)..string.format("%02d", (year-2000)).."_"..string.format("%02d", h)..string.format("%02d", m)..string.format("%02d", s)..".bmp",false)
		end
		if Controls.check(pad, KEY_L) then
			z = z + 0.1
		end
		if Controls.check(pad, KEY_R) then
			z = z - 0.1
		end
		if Controls.check(pad, KEY_TOUCH) then
			tx, ty = Controls.readTouch()
			if oldtx == nil then
				oldtx = tx
				oldty = ty
			end
			if oldtx < tx then
				angleX = angleX - 0.1
			elseif oldtx > tx then
				angleX = angleX + 0.1
			end
			if oldty < ty then
				angleY = angleY - 0.1
			elseif oldty > ty then
				angleY = angleY + 0.1
			end
			oldtx = tx
			oldty = ty
		else
			oldtx = nil
			oldty = nil
		end
		cx, cy = Controls.readCirclePad()
		if cx < -30 then
			x = x + 0.1
		elseif cx > 30 then
			x = x - 0.1
		end
		if cy < -30 then
			y = y + 0.1
		elseif cy > 30 then
			y = y - 0.1
		end
	end
	
	-- CPU Speed management
	if Controls.check(pad, KEY_X) and not Controls.check(oldpad, KEY_X) then
		if cpu_speed == 268 then
			System.setCpuSpeed(804)
		else
			System.setCpuSpeed(268)
		end
		cpu_speed = System.getCpuSpeed()
	end
	
	-- Updating FPS counter
	if Timer.getTime(tmr) > 1000 then
		fps_sec = fps
		fps = 0
		Timer.reset(tmr)
	end
	
	-- Refreshing screen and oldpad
	Screen.flip()
	oldpad = pad
	
end